package ij.io;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import ij.*;
import ij.gui.*;
import ij.process.*;
import ij.util.StringSorter;


/** This is a dialog box used to imports raw 8, 16, 24 and 32-bit images. */
public class ImportDialog {
	private String fileName;
    private String directory;
	
    // default settings
    private static int choiceSelection = 0; // 8-bit
    private static int width = 512;
    private static int height = 512;
    private static int offset = 0;
    private static int nImages = 1;
    private static int gapBetweenImages = 0;
    private static boolean whiteIsZero = false;
    private static boolean intelByteOrder = false;
    private static boolean openAll = false;
    private static String[] types = {"8-bit", "16-bit Signed", "16-bit Unsigned",
    	"32-bit Integer", "32-bit Real", "24-bit RGB", "24-bit RGB Planar"};
	
    public ImportDialog(String fileName, String directory) {
        this.fileName = fileName;
        this.directory = directory;
		IJ.showStatus("Importing: " + fileName);
		IJ.register(ImportDialog.class);  // keep this class from being GC'd
	}

	boolean showDialog() {
		GenericDialog gd = new GenericDialog("Import...", IJ.getInstance());
		gd.addChoice("Image Type:", types, types[choiceSelection]);
		gd.addNumericField("Image Width:", width, 0);
		gd.addNumericField("Image Height:", height, 0);
		gd.addNumericField("Offset to First Image:", offset, 0);
		gd.addNumericField("Number of Images:", nImages, 0);
		gd.addNumericField("Gap Between Images:", gapBetweenImages, 0);
		gd.addCheckbox("White is Zero", whiteIsZero);
		gd.addCheckbox("Little-Endian Byte Order", intelByteOrder);
		gd.addCheckbox("Open All", openAll);
		gd.showDialog();
		if (gd.wasCanceled())
			return false;
		choiceSelection = gd.getNextChoiceIndex();
		width = (int)gd.getNextNumber();
		height = (int)gd.getNextNumber();
		offset = (int)gd.getNextNumber();
		nImages = (int)gd.getNextNumber();
		gapBetweenImages = (int)gd.getNextNumber();
		whiteIsZero = gd.getNextBoolean();
		intelByteOrder = gd.getNextBoolean();
		openAll = gd.getNextBoolean();
		return true;
	}
	
	/** Opens all the images in the directory. */
	void openAll(String[] list, FileInfo fi) {
		StringSorter.sort(list);
		ImageStack stack=null;
		ImagePlus imp=null;
		for (int i=0; i<list.length; i++) {
			fi.fileName = list[i];
			imp = new FileOpener(fi).open(false);
			if (imp==null)
				IJ.write(list[i] + ": unable to open");
			else {
				if (stack==null)
					stack = imp.createEmptyStack();	
				try {
					stack.addSlice(list[i], imp.getProcessor());
				}
				catch(OutOfMemoryError e) {
					IJ.outOfMemory("OpenAll");
					stack.trim();
					break;
				}
				IJ.showStatus((stack.getSize()+1) + ": " + list[i]);
			}
		}
		imp = new ImagePlus(list[0], stack);
		imp.show();
	}
	
	/** Opens the specified image. Does nothing if the dialog was canceled. */
	public void openImage() {
		if (!showDialog())
			return;
		FileInputStream in;
		String path = directory + fileName;
		File f = new File(path);
		String imageType = types[choiceSelection];

		FileInfo fi = new FileInfo();
		fi.fileFormat = fi.RAW;
		fi.fileName = fileName;
		fi.directory = directory;
		fi.width = width;
		fi.height = height;
		fi.offset = offset;
		fi.nImages = nImages;
		fi.gapBetweenImages = gapBetweenImages;
		fi.intelByteOrder = intelByteOrder;
		fi.whiteIsZero = whiteIsZero;
		if (imageType.equals("8-bit"))
			fi.fileType = FileInfo.GRAY8;
		else if (imageType.equals("16-bit Signed"))
			fi.fileType = FileInfo.GRAY16_SIGNED;
		else if (imageType.equals("16-bit Unsigned"))
			fi.fileType = FileInfo.GRAY16_UNSIGNED;
		else if (imageType.equals("32-bit Integer"))
			fi.fileType = FileInfo.GRAY32_INT;
		else if (imageType.equals("32-bit Real"))
			fi.fileType = FileInfo.GRAY32_FLOAT;
		else if (imageType.equals("24-bit RGB"))
			fi.fileType = FileInfo.RGB;
		else if (imageType.equals("24-bit RGB Planar"))
			fi.fileType = FileInfo.RGB_PLANAR;
		else
			return;
			
		if (IJ.debugMode) IJ.write("ImportDialog: "+fi);
		if (openAll) {
			String[] list = new File(directory).list();
			if (list==null)
				return;
			openAll(list, fi);
		} else {
			FileOpener fo = new FileOpener(fi);
			fo.open();
		}
	}

}